# Copyright (c) 2013 Hortonworks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re
from savanna.openstack.common import log as logging
from savanna.utils import remote
import six

LOG = logging.getLogger(__name__)


class HadoopServer:
    _master_ip = None

    def __init__(self, instance, node_group):
        self.instance = instance
        self.node_group = node_group
        self._ssh = self._connect_to_vm()

    def _connect_to_vm(self):
        LOG.info(
            'Connecting to VM: {0}'.format(self.instance.management_ip))
        return self.instance.remote.ssh_connection()

    def provision_ambari(self, ambari_server_ip):
        self.install_rpms()
        if 'AMBARI_SERVER' in self.node_group.components:
            self._setup_and_start_ambari_server()

        if 'AMBARI_AGENT' in self.node_group.components:
            self._setup_and_start_ambari_agent(ambari_server_ip)

    def install_rpms(self):
        LOG.info(
            "{0}: Installing rpm's ...".format(self.instance.hostname))

        #TODO(jspeidel): based on image type, use correct command
        rpm_cmd = 'rpm -Uvh http://s3.amazonaws.com/dev.hortonworks' \
                  '.com/AMBARI.b6-1.x/repos/centos6/AMBARI.b6-1.x-1.el6' \
                  '.noarch.rpm'
        self._execute_on_vm(rpm_cmd)
        self._execute_on_vm('yum -y install epel-release')

    def _setup_and_start_ambari_server(self):
        LOG.info(
            '{0}: Installing ambari-server ...'.format(self.instance.hostname))
        self._execute_on_vm('yum -y install ambari-server')

        LOG.info('Running Ambari Server setup ...')
        self._execute_on_vm_interactive('ambari-server setup',
                                        DefaultPromptMatcher(
                                            "Ambari Server 'setup' finished "
                                            "successfully",
                                            LOG))

        LOG.info('Starting Ambari ...')
        self._execute_on_vm('ambari-server start')

    def _setup_and_start_ambari_agent(self, ambari_server_ip):
        LOG.info(
            '{0}: Installing Ambari Agent ...'.format(self.instance.hostname))

        self._execute_on_vm('yum -y install ambari-agent')
        LOG.debug(
            '{0}: setting master-ip: {1} in ambari-agent.ini'.format(
                self.instance.hostname, ambari_server_ip))
        self._replace_str_in_remote_file(
            '/etc/ambari-agent/conf/ambari-agent.ini', 'localhost',
            ambari_server_ip)

        LOG.info(
            '{0}: Starting Ambari Agent ...'.format(self.instance.hostname))
        self._execute_on_vm('ambari-agent start')

    def _configure_ganglia(self, ganglia_server_ip):
        #TODO(John): the set of files to update is now dependent on which
        # components are deployed on a host
        #TODO(jspeidel): so we these calls should be based on configuration
        LOG.debug(
            '{0}: Updating Ganglia host configuration ...'.format(
                self.instance.hostname))

        # slave config
        #TODO(jspeidel): set MASTER_SLAVE for master where only one node is
        # deployed
        if self._is_ganglia_slave() or self._is_ganglia_master():
            self._replace_str_in_remote_file(
                '/etc/ganglia/hdp/HDPSlaves/conf.d/gmond.slave.conf',
                'host = {0}'.format(self.instance.hostname),
                'host = {0}'.format(ganglia_server_ip))
            self._replace_str_in_remote_file(
                '/etc/ganglia/hdp/HDPJobTracker/conf.d/gmond.slave.conf',
                'host = {0}'.format(self.instance.hostname),
                'host = {0}'.format(ganglia_server_ip))
            self._replace_str_in_remote_file(
                '/etc/ganglia/hdp/HDPNameNode/conf.d/gmond.slave.conf',
                'host = {0}'.format(self.instance.hostname),
                'host = {0}'.format(ganglia_server_ip))
            self._replace_str_in_remote_file(
                '/etc/ganglia/hdp/HDPHBaseMaster/conf.d/gmond.slave.conf',
                'host = {0}'.format(self.instance.hostname),
                'host = {0}'.format(ganglia_server_ip))

        #master config
        if self._is_ganglia_master():
            self._replace_str_in_remote_file(
                '/etc/ganglia/hdp/HDPSlaves/conf.d/gmond.master.conf',
                'bind = {0}'.format(self.instance.hostname), '')
            self._replace_str_in_remote_file(
                '/etc/ganglia/hdp/HDPJobTracker/conf.d/gmond.master.conf',
                'bind = {0}'.format(self.instance.hostname), '')
            self._replace_str_in_remote_file(
                '/etc/ganglia/hdp/HDPNameNode/conf.d/gmond.master.conf',
                'bind = {0}'.format(self.instance.hostname), '')
            #TODO(jspeidel): appears only to be necessary if hbase is installed
        #            self._replace_str_in_remote_file(self._ssh,
        # '/etc/ganglia/hdp/HDPHBaseMaster/conf.d/gmond.master.conf',
        #                                             'bind = {0}'.format(
        # self.instance.fqdn), '')

        # gangliaClusters.conf
        self._replace_str_in_remote_file(
            '/usr/libexec/hdp/ganglia/gangliaClusters.conf',
            self.instance.fqdn, ganglia_server_ip)

        # update puppet templates and shell scripts because they generate
        # configs that are used after restart
        # gangliaClusters.conf template
        #TODO(jspeidel): modify file where prop "ganglia_server_host" is set
        self._replace_str_in_remote_file(
            '/var/lib/ambari-agent/puppet/modules/hdp-ganglia/templates'
            '/gangliaClusters.conf.erb',
            '<%=scope.function_hdp_host("ganglia_server_host")%>',
            ganglia_server_ip)

        # gmondLib.sh This script generates the master and slave configs
        #TODO(jspeidel): combine into one call.  Pass map of old/new values
        self._replace_str_in_remote_file(
            '/var/lib/ambari-agent/puppet/modules/hdp-ganglia/files/gmondLib'
            '.sh',
            'bind = ${gmondMasterIP}', '')
        self._replace_str_in_remote_file(
            '/var/lib/ambari-agent/puppet/modules/hdp-ganglia/files/gmondLib'
            '.sh',
            'host = ${gmondMasterIP}', 'host = {0}'.format(ganglia_server_ip))
        self._replace_str_in_remote_file(
            '/usr/libexec/hdp/ganglia/gmondLib.sh',
            'bind = ${gmondMasterIP}', '')
        self._replace_str_in_remote_file(
            '/usr/libexec/hdp/ganglia/gmondLib.sh',
            'host = ${gmondMasterIP}', 'host = {0}'.format(ganglia_server_ip))

    def update_hosts_file(self, hosts):
        # read the hosts file
        data = remote.read_file_from(self._ssh.open_sftp(), '/etc/hosts')
        output = six.StringIO()
        for host in hosts:
            output.write('{0}   {1}\n'.format(host.instance.internal_ip,
                                              host.instance.fqdn))
        output.write('\n')

        # add the previous file contents
        output.write(data)

        # write the file back
        LOG.debug("updating hosts file with the following:")
        LOG.debug(output.getvalue())
        remote.write_file_to(self._ssh.open_sftp(), '/etc/hosts',
                             output.getvalue())

    def _replace_str_in_remote_file(self, filename, origStr, newStr):
        s = ''

        client = self._ssh.open_sftp()
        #check that file exists
        try:
            client.stat(filename)
            client.close()
        except IOError:
            LOG.debug(
                "File to edit doesn't exist: {0}".format(filename))
            return

        # copy the file locally
        data = remote.read_file_from(self._ssh.open_sftp(), filename)
        # read thru lines and replace origStr with newStr
        for line in six.StringIO(data):
            if origStr in line:
                if newStr != '':
                    s += line.replace(origStr, newStr)
            else:
                s += line

        # write the file back out
        remote.write_file_to(self._ssh.open_sftp(), filename, s)

    def _log(self, buf):
        LOG.debug(buf)

    def _execute_on_vm_interactive(self, cmd, matcher):
        LOG.debug(
            "{0}: Executing interactive remote command '{1}'".format(
                self.instance.hostname, cmd))

        buf = ''
        all_output = ''
        channel = self._ssh.invoke_shell()
        try:
            channel.send(cmd + '\n')
            while not matcher.is_eof(buf):
                buf += channel.recv(4096)
                response = matcher.get_response(buf)
                if response is not None:
                    channel.send(response + '\n')
                    all_output += buf
                    buf = ''
        finally:
            channel.close()
            self._log(all_output)
            self._log(buf)

    def _execute_on_vm(self, cmd):
        LOG.debug("{0}: Executing remote command '{1}'".format(
            self.instance.hostname, cmd))
        LOG.debug(
            'Executing using instance: id = {0}, hostname = {1}'.format(
                self.instance.instance_id,
                self.instance.hostname))
        remote.execute_command(self._ssh, cmd)

    def _is_component_available(self, component):
        return component in self.node_group.components

    def _is_ganglia_master(self):
        return self._is_component_available('GANGLIA_SERVER')

    def _is_ganglia_slave(self):
        return self._is_component_available('GANGLIA_MONITOR')


class DefaultPromptMatcher():
    prompt_pattern = re.compile('(.*\()(.)(\)\?\s*$)', re.DOTALL)

    def __init__(self, terminal_token, logger):
        self.eof_token = terminal_token
        self.logger = logger

    def get_response(self, s):
        match = self.prompt_pattern.match(s)
        if match:
            response = match.group(2)
            LOG.debug(
                "Returning response '{0}' for prompt '{1}'".format(
                    response, s.rstrip().rsplit('\n', 1)[-1]))
            return response
        else:
            return None

    def is_eof(self, s):
        eof = self.eof_token in s
        if eof:
            LOG.debug('Returning eof = True')
        return eof



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import setuptools

from savanna.openstack.common import setup as common_setup

requires = common_setup.parse_requirements()
depend_links = common_setup.parse_dependency_links()
project = 'savanna'

setuptools.setup(
    name=project,
    version=common_setup.get_version(project, '0.2'),
    description='Savanna project',
    author='OpenStack',
    author_email='openstack-dev@lists.openstack.org',
    url='https://savanna.readthedocs.org',
    classifiers=[
        'Environment :: OpenStack',
        'Intended Audience :: Information Technology',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
    ],
    license='Apache Software License',
    cmdclass=common_setup.get_cmdclass(),
    packages=setuptools.find_packages(exclude=['bin']),
    package_data={'savanna': [
        'plugins/vanilla/resources/*.xml',
        'swift/resources/*.xml',
        'tests/unit/resources/*.xml',
        'tests/unit/resources/*.txt',
    ]},
    install_requires=requires,
    dependency_links=depend_links,
    setup_requires=['setuptools-git>=0.4'],
    include_package_data=True,
    test_suite='nose.collector',
    scripts=[
        'bin/savanna-api',
        'bin/savanna-db-manage',
    ],
    py_modules=[],
    data_files=[
        ('share/savanna',
         [
             'etc/savanna/savanna.conf.sample',
             'etc/savanna/savanna.conf.sample-full',
         ]),
    ],
)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import telnetlib

from savanna.tests.integration import base
import savanna.tests.integration.configs.parameters as param


def empty_object_id(expr):
    return '' if expr else param.IMAGE_ID


class ImageRegistryCrudTest(base.ITestCase):

    def _check_images_list_accessible(self):
        self.get_object(self.url_images, empty_object_id(True), 200)

    def _set_description_username(self, description, username):
        url = self.url_images + '/' + param.IMAGE_ID
        body = dict(
            description=description,
            username=username
        )
        data = self.post_object(url, body, 202)
        return data

    def _send_rest_on_tag(self, url_part, tag_name):
        url = self.url_images + '/' + param.IMAGE_ID + url_part
        tag = [tag_name]
        body = dict(tags=tag)
        data = self.post_object(url, body, 202)
        return data

    def _get_image_description(self):
        url = self.url_images + '/'
        data = self.get_object(url, empty_object_id(False), 200)
        return data

    def _set_and_compare_tags(self, tag):
        data = self._get_image_description()
        tag_data = data['image']['tags']
        if tag not in data['image']['tags']:
            data = self._send_rest_on_tag('/tag', tag)
            tag_data.append(tag)
        self.assertItemsEqual(data['image']['tags'], tag_data,
                              'tags comparison has failed')
        return tag_data

    def _delete_tag_and_check(self, tag):
        data = self._get_image_description()
        untag_data = data['image']['tags']
        data = self._send_rest_on_tag('/untag', tag)
        untag_data.remove(tag)
        self.assertItemsEqual(data['image']['tags'], untag_data,
                              'tags comparison has failed')

    def _get_image_by_tags(self, tag_name):
        url = self.url_images + '?tags=' + tag_name
        data = self.get_object(url, empty_object_id(True), 200)
        if param.IMAGE_ID not in [img['id'] for img in data['images']]:
            self.fail('image by tag \'%s\' not found' % tag_name)

    def _get_image_by_delete_tags(self, tag_name):
        url = self.url_images + '?tags=' + tag_name
        data = self.get_object(url, empty_object_id(True), 200)
        if param.IMAGE_ID in [img['id'] for img in data['images']]:
            self.fail('image tag \'%s\' is not deleted' % tag_name)

    def setUp(self):
        super(ImageRegistryCrudTest, self).setUp()
        telnetlib.Telnet(self.host, self.port)

    def test_image_registry(self):
        """This test checks image registry work
        """
        username = 'ubuntu'
        tag1_name = 'animal'
        tag2_name = 'dog'
        description = 'working image'

        self._check_images_list_accessible()

        data = self._set_description_username(description, username)
        self.assertEquals(data['image']['description'], description)
        self.assertEquals(data['image']['username'], username)

        try:
            self._set_and_compare_tags(tag1_name)
            tag_data = self._set_and_compare_tags(tag2_name)

            self._get_image_by_tags(tag1_name)
            self._get_image_by_tags(tag2_name)

            data = self._get_image_description()
            self.assertEquals(data['image']['status'], 'ACTIVE')
            self.assertEquals(data['image']['username'], username)
            self.assertItemsEqual(data['image']['tags'], tag_data)
            self.assertEquals(data['image']['description'], description)
            self.assertEquals(data['image']['id'], param.IMAGE_ID)

            self._delete_tag_and_check(tag1_name)
            self._delete_tag_and_check(tag2_name)

            self._get_image_by_delete_tags(tag1_name)
            self._get_image_by_delete_tags(tag2_name)

            self._check_images_list_accessible()

        except Exception as e:
            self._send_rest_on_tag('/untag', tag1_name)
            self._send_rest_on_tag('/untag', tag2_name)
            self.fail(str(e))



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from savanna.openstack.common import log as logging
from savanna.plugins import provisioning as p
from savanna.plugins.vanilla import config_helper as c_helper
from savanna.plugins.vanilla import exceptions as ex
from savanna.plugins.vanilla import run_scripts as run
from savanna.plugins.vanilla import scaling as sc
from savanna.plugins.vanilla import utils
from savanna.utils import crypto


LOG = logging.getLogger(__name__)


class VanillaProvider(p.ProvisioningPluginBase):
    def __init__(self):
        self.processes = {
            "HDFS": ["namenode", "datanode", "secondarynamenode"],
            "MapReduce": ["tasktracker", "jobtracker"]
        }

    def get_plugin_opts(self):
        return []

    def setup(self, conf):
        self.conf = conf

    def get_title(self):
        return "Vanilla Apache Hadoop"

    def get_description(self):
        return (
            "This plugin provides an ability to launch vanilla Apache Hadoop "
            "cluster without any management consoles.")

    def get_versions(self):
        return ['1.1.2']

    def get_configs(self, hadoop_version):
        return c_helper.get_plugin_configs()

    def get_node_processes(self, hadoop_version):
        return self.processes

    def validate(self, cluster):
        nn_count = sum([ng.count for ng
                        in utils.get_node_groups(cluster, "namenode")])
        if nn_count is not 1:
            raise ex.NotSingleNameNodeException(nn_count)

        jt_count = sum([ng.count for ng
                        in utils.get_node_groups(cluster, "jobtracker")])

        if jt_count not in [0, 1]:
            raise ex.NotSingleJobTrackerException(jt_count)

        tt_count = sum([ng.count for ng
                        in utils.get_node_groups(cluster, "tasktracker")])
        if jt_count is 0 and tt_count > 0:
            raise ex.TaskTrackersWithoutJobTracker()

    def update_infra(self, cluster):
        pass

    def configure_cluster(self, cluster):
        self._extract_configs(cluster)
        self._push_configs_to_nodes(cluster)
        self._write_hadoop_user_keys(cluster.private_key,
                                     utils.get_instances(cluster))

    def start_cluster(self, cluster):
        nn_instance = utils.get_namenode(cluster)
        datanodes = utils.get_datanodes(cluster)
        jt_instance = utils.get_jobtracker(cluster)
        tasktrackers = utils.get_tasktrackers(cluster)

        with nn_instance.remote as remote:
            run.format_namenode(remote)
            run.start_process(remote, "namenode")

        snns = utils.get_secondarynamenodes(cluster)
        if snns:
            for snn in snns:
                run.start_process(snn.remote, "secondarynamenode")
        for dn in datanodes:
            run.start_process(dn.remote, "datanode")
        LOG.info("HDFS service at '%s' has been started",
                 nn_instance.hostname)

        if jt_instance:
            run.start_process(jt_instance.remote, "jobtracker")
            for tt in tasktrackers:
                run.start_process(tt.remote, "tasktracker")
            LOG.info("MapReduce service at '%s' has been started",
                     jt_instance.hostname)

        LOG.info('Cluster %s has been started successfully' % cluster.name)
        self._set_cluster_info(cluster)

    def _extract_configs(self, cluster):
        nn = utils.get_namenode(cluster)
        jt = utils.get_jobtracker(cluster)

        for ng in cluster.node_groups:
            ng.extra = {
                'xml': c_helper.generate_xml_configs(ng.configuration,
                                                     ng.storage_paths,
                                                     nn.hostname,
                                                     jt.hostname
                                                     if jt else None),
                'setup_script': c_helper.generate_setup_script(
                    ng.storage_paths,
                    c_helper.extract_environment_confs(ng.configuration)
                )
            }

    def decommission_nodes(self, cluster, instances):
        tts = utils.get_tasktrackers(cluster)
        dns = utils.get_datanodes(cluster)
        decommission_dns = False
        decommission_tts = False

        for i in instances:
            if 'datanode' in i.node_group.node_processes:
                dns.remove(i)
                decommission_dns = True
            if 'tasktracker' in i.node_group.node_processes:
                tts.remove(i)
                decommission_tts = True

        nn = utils.get_namenode(cluster)
        jt = utils.get_jobtracker(cluster)

        if decommission_tts:
            sc.decommission_tt(jt, instances, tts)
        if decommission_dns:
            sc.decommission_dn(nn, instances, dns)

    def validate_scaling(self, cluster, existing, additional):
        self._validate_existing_ng_scaling(cluster, existing)
        self._validate_additional_ng_scaling(cluster, additional)

    def scale_cluster(self, cluster, instances):
        self._extract_configs(cluster)
        self._push_configs_to_nodes(cluster, instances=instances)
        self._write_hadoop_user_keys(cluster.private_key,
                                     instances)
        run.refresh_nodes(utils.get_namenode(cluster).remote, "dfsadmin")
        jt = utils.get_jobtracker(cluster)
        if jt:
            run.refresh_nodes(jt.remote, "mradmin")

        for i in instances:
            with i.remote as remote:
                if "datanode" in i.node_group.node_processes:
                    run.start_process(remote, "datanode")

                if "tasktracker" in i.node_group.node_processes:
                    run.start_process(remote, "tasktracker")

    def _push_configs_to_nodes(self, cluster, instances=None):
        if instances is None:
            instances = utils.get_instances(cluster)

        for inst in instances:
            files = {
                '/etc/hadoop/core-site.xml': inst.node_group.extra['xml'][
                    'core-site'],
                '/etc/hadoop/mapred-site.xml': inst.node_group.extra['xml'][
                    'mapred-site'],
                '/etc/hadoop/hdfs-site.xml': inst.node_group.extra['xml'][
                    'hdfs-site'],
                '/tmp/savanna-hadoop-init.sh': inst.node_group.extra[
                    'setup_script']
            }
            with inst.remote as r:
                r.execute_command(
                    'sudo chown -R $USER:$USER /etc/hadoop'
                )
                r.write_files_to(files)
                r.execute_command(
                    'sudo chmod 0500 /tmp/savanna-hadoop-init.sh'
                )
                r.execute_command(
                    'sudo /tmp/savanna-hadoop-init.sh '
                    '>> /tmp/savanna-hadoop-init.log 2>&1')

        nn = utils.get_namenode(cluster)
        jt = utils.get_jobtracker(cluster)

        with nn.remote as r:
            r.write_file_to('/etc/hadoop/dn.incl', utils.
                            generate_fqdn_host_names(
                            utils.get_datanodes(cluster)))
        if jt:
            with jt.remote as r:
                r.write_file_to('/etc/hadoop/tt.incl', utils.
                                generate_fqdn_host_names(
                                utils.get_tasktrackers(cluster)))

    def _set_cluster_info(self, cluster):
        nn = utils.get_namenode(cluster)
        jt = utils.get_jobtracker(cluster)

        info = cluster.info

        if jt and jt.management_ip:
            info['MapReduce'] = {
                'Web UI': 'http://%s:50030' % jt.management_ip
            }
        if nn and nn.management_ip:
            info['HDFS'] = {
                'Web UI': 'http://%s:50070' % nn.management_ip
            }

    def _write_hadoop_user_keys(self, private_key, instances):
        public_key = crypto.private_key_to_public_key(private_key)

        files = {
            'id_rsa': private_key,
            'authorized_keys': public_key
        }

        mv_cmd = 'sudo mkdir -p /home/hadoop/.ssh/; ' \
                 'sudo mv id_rsa authorized_keys /home/hadoop/.ssh ; ' \
                 'sudo chown -R hadoop:hadoop /home/hadoop/.ssh; ' \
                 'sudo chmod 600 /home/hadoop/.ssh/{id_rsa,authorized_keys}'

        for instance in instances:
            with instance.remote as remote:
                remote.write_files_to(files)
                remote.execute_command(mv_cmd)

    def _get_scalable_processes(self):
        return ["datanode", "tasktracker"]

    def _validate_additional_ng_scaling(self, cluster, additional):
        jt = utils.get_jobtracker(cluster)
        scalable_processes = self._get_scalable_processes()

        for ng in additional:
            if not set(ng.node_processes).issubset(scalable_processes):
                raise ex.NodeGroupCannotBeScaled(
                    ng.name, "Vanilla plugin cannot scale nodegroup"
                             " with processes: " +
                             ' '.join(ng.node_processes))
            if not jt and 'tasktracker' in ng.node_processes:
                raise ex.NodeGroupCannotBeScaled(
                    ng.name, "Vanilla plugin cannot scale node group with "
                             "processes which have no master-processes run "
                             "in cluster")

    def _validate_existing_ng_scaling(self, cluster, existing):
        ng_names = existing.copy()
        scalable_processes = self._get_scalable_processes()
        dn_to_delete = 0
        for ng in cluster.node_groups:
            if ng.name in existing:
                del ng_names[ng.name]
                if ng.count > existing[ng.name] and "datanode" in \
                        ng.node_processes:
                    dn_to_delete += ng.count - existing[ng.name]
                if not set(ng.node_processes).issubset(scalable_processes):
                    raise ex.NodeGroupCannotBeScaled(
                        ng.name, "Vanilla plugin cannot scale nodegroup"
                                 " with processes: " +
                                 ' '.join(ng.node_processes))

        dn_amount = len(utils.get_datanodes(cluster))
        rep_factor = c_helper.determine_cluster_config(cluster,
                                                       "dfs.replication")

        if dn_to_delete > 0 and dn_amount - dn_to_delete < rep_factor:
            raise Exception("Vanilla plugin cannot shrink cluster because "
                            "it would be not enough nodes for replicas "
                            "(replication factor is %s )" % rep_factor)

        if len(ng_names) != 0:
            raise ex.NodeGroupsDoNotExist(ng_names.keys())



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import netaddr

from oslo.config import cfg

from savanna.utils.openstack import nova

CONF = cfg.CONF

cluster_node_opts = [
    cfg.BoolOpt('use_floating_ips',
                default=True,
                help='When set to false, Savanna uses only internal IP of VMs.'
                     ' When set to true, Savanna expects OpenStack to auto-'
                     'assign floating IPs to cluster nodes. Internal IPs will '
                     'be used for inter-cluster communication, while floating '
                     'ones will be used by Savanna to configure nodes. Also '
                     'floating IPs will be exposed in service URLs.'),
    cfg.StrOpt('node_domain',
               default='novalocal',
               help="The suffix of the node's FQDN. In nova-network that is "
                    "dhcp_domain config parameter")
]

CONF.register_opts(cluster_node_opts)


# NOTE(slukjanov): https://blueprints.launchpad.net/savanna?searchtext=ip
def init_instances_ips(instance, server):
    """Extracts internal and management ips.

    As internal ip will be used the first ip from the nova networks CIDRs.
    If use_floating_ip flag is set than management ip will be the first
    non-internal ip.
    """
    if instance.internal_ip and instance.management_ip:
        return True

    for network_label in server.networks:
        nova_network = nova.client().networks.find(label=network_label)
        network = netaddr.IPNetwork(nova_network.cidr)
        for ip in server.networks[network_label]:
            if netaddr.IPAddress(ip) in network:
                instance.internal_ip = instance.internal_ip or ip
            else:
                instance.management_ip = instance.management_ip or ip

    if not CONF.use_floating_ips:
        instance.management_ip = instance.internal_ip

    return instance.internal_ip and instance.management_ip



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools

import jsonschema

from savanna import exceptions as ex
import savanna.openstack.common.exception as os_ex
from savanna.utils import api as u
from savanna.utils import api_validator


def validate(schema, *validators):
    def decorator(func):
        @functools.wraps(func)
        def handler(*args, **kwargs):
            request_data = u.request_data()
            try:
                if schema:
                    validator = api_validator.ApiValidator(schema)
                    validator.validate(request_data)
                if validators:
                    for validator in validators:
                        validator(**kwargs)
            except jsonschema.ValidationError as e:
                e.code = "VALIDATION_ERROR"
                return u.bad_request(e)
            except ex.SavannaException as e:
                return u.bad_request(e)
            except os_ex.MalformedRequestBody as e:
                e.code = "MALFORMED_REQUEST_BODY"
                return u.bad_request(e)
            except Exception as e:
                return u.internal_error(
                    500, "Error occurred during validation", e)

            return func(*args, **kwargs)

        return handler

    return decorator


def check_exists(get_func, *id_prop, **get_args):
    def decorator(func):
        @functools.wraps(func)
        def handler(*args, **kwargs):
            if id_prop and not get_args:
                get_args['id'] = id_prop[0]

            get_kwargs = {}
            for get_arg in get_args:
                get_kwargs[get_arg] = kwargs[get_args[get_arg]]

            obj = None
            try:
                obj = get_func(**get_kwargs)
            except Exception as e:
                if 'notfound' not in e.__class__.__name__.lower():
                    raise e
            if obj is None:
                e = ex.NotFoundException(get_kwargs,
                                         'Object with %s not found')
                return u.not_found(e)

            return func(*args, **kwargs)

        return handler

    return decorator



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import savanna.openstack.common.importutils as importutils

_CONF = importutils.try_import('savanna.tests.integration.configs.config')


def _get_conf(key, default):
    return getattr(_CONF, key) if _CONF and hasattr(_CONF, key) else default

OS_USERNAME = _get_conf('OS_USERNAME', 'admin')
OS_PASSWORD = _get_conf('OS_PASSWORD', 'password')
OS_TENANT_NAME = _get_conf('OS_TENANT_NAME', 'admin')
OS_AUTH_URL = _get_conf('OS_AUTH_URL', 'http://192.168.1.1:35357/v2.0/')

SAVANNA_HOST = _get_conf('SAVANNA_HOST', '192.168.1.1')
SAVANNA_PORT = _get_conf('SAVANNA_PORT', '8386')

IMAGE_ID = _get_conf('IMAGE_ID', '42abc')
FLAVOR_ID = _get_conf('FLAVOR_ID', '42')

NODE_USERNAME = _get_conf('NODE_USERNAME', 'username')

CLUSTER_NAME_CRUD = _get_conf('CLUSTER_NAME_CRUD', 'cluster-crud')
CLUSTER_NAME_HADOOP = _get_conf('CLUSTER_NAME_HADOOP', 'cluster-hadoop')
CLUSTER_NAME_SWIFT = _get_conf('CLUSTER_NAME_SWIFT', 'cluster-swift')
CLUSTER_NAME_SCALING = _get_conf('CLUSTER_NAME_SCALING', 'cluster-scaling')

TIMEOUT = _get_conf('TIMEOUT', 15)

HADOOP_VERSION = _get_conf('HADOOP_VERSION', '1.1.2')
HADOOP_DIRECTORY = _get_conf('HADOOP_DIRECTORY', '/usr/share/hadoop')
HADOOP_LOG_DIRECTORY = _get_conf('HADOOP_LOG_DIRECTORY',
                                 '/mnt/log/hadoop/hadoop/userlogs')

SSH_KEY = _get_conf('SSH_KEY', 'jenkins')
PATH_TO_SSH = _get_conf('PATH_TO_SSH', '/home/user/.ssh/id_rsa')

PLUGIN_NAME = _get_conf('PLUGIN_NAME', 'vanilla')

NAMENODE_CONFIG = _get_conf('NAMENODE_CONFIG', {})
JOBTRACKER_CONFIG = _get_conf('JOBTRACKER_CONFIG', {})
DATANODE_CONFIG = _get_conf('DATANODE_CONFIG', {})
TASKTRACKER_CONFIG = _get_conf('TASKTRACKER_CONFIG', {})
GENERAL_CONFIG = _get_conf('GENERAL_CONFIG', {})
CLUSTER_HDFS_CONFIG = _get_conf('CLUSTER_HDFS_CONFIG', {})
CLUSTER_MAPREDUCE_CONFIG = _get_conf('CLUSTER_MAPREDUCE_CONFIG', {})

JT_PORT = _get_conf('JT_PORT', 50030)
NN_PORT = _get_conf('NN_PORT', 50070)
TT_PORT = _get_conf('TT_PORT', 50060)
DN_PORT = _get_conf('DN_PORT', 50075)

ENABLE_SWIFT_TESTS = _get_conf('ENABLE_SWIFT_TESTS', True)



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import telnetlib
import time

from savanna.tests.integration import base
import savanna.tests.integration.configs.parameters as param


class TestHadoop(base.ITestCase):

    def setUp(self):
        super(TestHadoop, self).setUp()
        telnetlib.Telnet(self.host, self.port)
        self.create_node_group_templates()

    def _hadoop_testing(self, cluster_tmpl_body):
        cl_tmpl_id = None
        cluster_id = None
        try:
            cl_tmpl_id = self.get_object_id(
                'cluster_template', self.post_object(self.url_cl_tmpl,
                                                     cluster_tmpl_body, 202))
            cluster_id = self.create_cluster_using_ngt_and_get_id(
                cl_tmpl_id, param.CLUSTER_NAME_HADOOP)
            ip_instances = self.get_instances_ip_and_node_processes_list(
                cluster_id)
            namenode_ip = None
            node_count = 0
            try:
                clstr_info = self.get_namenode_ip_and_tt_dn_count(ip_instances)
                namenode_ip = clstr_info['namenode_ip']
                node_count = clstr_info['node_count']
                self.await_active_workers_for_namenode(clstr_info)
            except Exception as e:
                self.fail(str(e))
            try:
                for key in ip_instances:
                    self.transfer_script_to_node(key)
            except Exception as e:
                self.fail('failure in transfer script: ' + str(e))
            try:
                self.execute_command(
                    namenode_ip, './script.sh pi -nc %s -hv %s -hd %s'
                                 % (node_count, param.HADOOP_VERSION,
                                    param.HADOOP_DIRECTORY))
            except Exception as e:
                print(self.read_file_from(namenode_ip,
                                          '/tmp/outputTestMapReduce/log.txt'))
                self.fail(
                    'run pi script has failed: '
                    + str(e))
            try:
                job_name = self.execute_command(
                    namenode_ip, './script.sh gn -hd %s'
                                 % param.HADOOP_DIRECTORY)[1]
                if job_name == 'JobId':
                    self.fail()
            except Exception as e:
                self.fail('fail in get job name: ' + str(e))

            for key, value in ip_instances.items():
                if 'datanode' in value or 'tasktracker' in value:
                    self.assertEquals(
                        self.execute_command(
                            key, './script.sh ed -jn %s -hld %s'
                                 % (job_name[:-1],
                                    param.HADOOP_LOG_DIRECTORY))[0], 0,
                        msg='fail in check run job in worker nodes: ')

            try:
                self.assertEquals(
                    self.execute_command(
                        namenode_ip, './script.sh mr -hv %s -hd %s'
                                     % (param.HADOOP_VERSION,
                                        param.HADOOP_DIRECTORY))[0], 0)
            except Exception as e:
                print(self.read_file_from(namenode_ip,
                                          '/tmp/outputTestMapReduce/log.txt'))
                self.fail('run hdfs script is failure: ' + str(e))
        except Exception as e:
            self.fail(str(e))

        finally:
            self.del_object(self.url_cluster_with_slash, cluster_id, 204)
            time.sleep(5)
            self.del_object(self.url_cl_tmpl_with_slash, cl_tmpl_id, 204)

    def test_hadoop_single_master(self):
        """This test checks hadoop work
        """
        node_list = {self.id_jt_nn: 1, self.id_tt_dn: 1}

        cl_tmpl_body = self.make_cluster_template('cl-tmpl-for-hadoop-test',
                                                  node_list)
        self._hadoop_testing(cl_tmpl_body)

    def tearDown(self):
        self.delete_node_group_templates()



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

if __name__ == '__main__':

    # open file 'hadoop-job-file.txt' for write and read
    f = open('hadoop-job-file.txt', 'wr')

    s = '1234567890'

    i = 0
    j = 0

    # strings number in file
    str_number = 1000000

    # repetitions number 's' in each string of file
    rep_number = 10

    while i < str_number:
        while j < rep_number:
            f.write(s)
            j += 1
        f.write('\n')
        i += 1
        j = 0

    f.close()



# Copyright (c) 2013 Hortonworks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import requests
from savanna import context
from savanna.openstack.common import jsonutils as json
from savanna.openstack.common import log as logging
from savanna.plugins.hdp import blueprintprocessor as bp
from savanna.plugins.hdp import clusterspec
from savanna.plugins.hdp import configprovider as cfg
from savanna.plugins.hdp import hadoopserver as h
from savanna.plugins.hdp import savannautils as s
from savanna.plugins import provisioning as p

LOG = logging.getLogger(__name__)


class AmbariPlugin(p.ProvisioningPluginBase):
    def __init__(self):
        self.cluster_name_to_ambari_host_mapping = {}
        self.default_config = self._get_default_cluster_configuration()

    def create_cluster(self, cluster, cluster_template):

        if cluster_template is None:
            raise ValueError('must supply cluster template')

        cluster_spec = clusterspec.ClusterSpec(cluster_template,
                                               cluster=cluster)

        hosts = self._get_servers(cluster)

        ambari_host = self._determine_host_for_server_component(
            'AMBARI_SERVER', cluster_spec, hosts)
        self.cluster_name_to_ambari_host_mapping[cluster.name] = ambari_host

        servers = []
        for host in hosts:
            servers.append(
                h.HadoopServer(host, cluster_spec.node_groups[host.role]))

        provisioned = self._provision_cluster(cluster.name, cluster_spec,
                                              ambari_host, servers)
        if provisioned:
            #self._update_server_hosts_files(servers)
            installed = self._install_services(cluster.name, ambari_host)
            if installed:
                LOG.info("Install of Hadoop stack successful.")
                # still need ssh to configure ganglia.
                ganglia_server_ip = self._determine_host_for_server_component(
                    'GANGLIA_SERVER',
                    cluster_spec, hosts).internal_ip
                self.update_ganglia_configurations(ganglia_server_ip, servers)
                # add service urls
                self._set_cluster_info(cluster, cluster_spec, hosts)
            else:
                LOG.warning("Install of Hadoop stack failed.")
        else:
            LOG.warning("Provisioning of Hadoop cluster failed.")

    def _get_servers(self, cluster):
        servers = []
        if hasattr(cluster, 'node_groups') and cluster.node_groups is not None:
            # code for a savanna cluster object
            for node_group in cluster.node_groups:
                for server in node_group.instances:
                    setattr(server, 'role', node_group.name)
                    setattr(server, 'node_processes',
                            node_group.node_processes)
                    servers.append(server)
        else:
            # cluster is actually a cloud context
            servers = cluster.instances

        return servers

    def get_node_processes(self, hadoop_version):
        #TODO(jmaron): use version information
        node_processes = {}
        for service in self.default_config.services:
            components = []
            for component in service.components:
                components.append(component.name)
            node_processes[service.name] = components

        return node_processes

    def convert(self, cluster_template, config):
        with open(config, 'r') as f:
            normalized_config = clusterspec.ClusterSpec(f.read()).normalize()

        #TODO(jspeidel):  can we get the name (first arg) from somewhere?

        cluster_template = s.convert(cluster_template, normalized_config)

        return cluster_template

    def update_infra(self, cluster):
        for node_group in cluster.node_groups:
            node_group.image = cluster.default_image_id

    def convert_props_to_template(self, props):
        raise NotImplementedError('not yet supported')

    def _spawn(self, func, args):
        context.spawn(func, args)

    def _add_cluster(self, ambari_public_ip, name):
        add_cluster_url = 'http://{0}:8080/api/v1/clusters/{1}'.format(
            ambari_public_ip, name)
        #TODO(jspeidel): get stack info from advanced config
        result = requests.post(add_cluster_url,
                               data='{"Clusters": {"version" : "HDP-1.2.0"}}',
                               auth=('admin', 'admin'))
        if result.status_code != 201:
            LOG.warning(
                'Create cluster command failed. {0}'.format(result.text))
            return False

        return True

    def _add_configurations_to_cluster(self, cluster_spec, ambari_public_ip,
                                       name):
        configs = cluster_spec.configurations
        add_configuration_url = 'http://{0}:8080/api/v1/clusters/{1}'.format(
            ambari_public_ip, name)
        body = {}
        clusters = {}
        body['Clusters'] = clusters
        for config_name in configs:
            config_body = {}
            clusters['desired_config'] = config_body
            config_body['type'] = config_name
            #TODO(jspeidel): hard coding for now
            config_body['tag'] = 'v1'
            config_body['properties'] = configs[config_name]
            result = requests.put(add_configuration_url, data=json.dumps(body),
                                  auth=('admin', 'admin'))
            if result.status_code != 200:
                LOG.warning(
                    'Set configuration command failed. {0}'.format(
                        result.text))
                return False

        return True

    def _add_services_to_cluster(self, cluster_spec, ambari_public_ip, name):
        services = cluster_spec.services
        add_service_url = 'http://{0}:8080/api/v1/clusters/{1}/services/{2}'
        for service in services:
            if service.name != 'AMBARI':
                result = requests.post(
                    add_service_url.format(ambari_public_ip, name,
                                           service.name),
                    auth=('admin', 'admin'))
                if result.status_code != 201:
                    LOG.warning(
                        'Create service command failed. {0}'.format(
                            result.text))
                    return False

        return True

    def _add_components_to_services(self, cluster_spec, ambari_public_ip,
                                    name):
        add_component_url = 'http://{0}:8080/api/v1/clusters/{1}/services/{' \
                            '2}/components/{3}'
        for service in cluster_spec.services:
            if service.name != 'AMBARI':
                for component in service.components:
                    result = requests.post(add_component_url.format(
                        ambari_public_ip, name, service.name, component.name),
                        auth=('admin', 'admin'))
                    if result.status_code != 201:
                        LOG.warning(
                            'Create component command failed. {0}'.format(
                                result.text))
                        return False

        return True

    def _add_hosts_and_components(self, cluster_spec, servers,
                                  ambari_public_ip, name):
        add_host_url = 'http://{0}:8080/api/v1/clusters/{1}/hosts/{2}'
        add_host_component_url = 'http://{0}:8080/api/v1/clusters/{' \
                                 '1}/hosts/{2}/host_components/{3}'
        for host in servers:
            hostname = host.instance.fqdn.lower()
            result = requests.post(
                add_host_url.format(ambari_public_ip, name, hostname),
                auth=('admin', 'admin'))
            if result.status_code != 201:
                LOG.warning(
                    'Create host command failed. {0}'.format(result.text))
                return False
                #TODO(jspeidel): is role really same as node group?
            node_group_name = host.node_group.name
            #TODO(jspeidel): ensure that node group exists
            node_group = cluster_spec.node_groups[node_group_name]
            for component in node_group.components:
                # don't add any AMBARI components
                if component.find('AMBARI') != 0:
                    result = requests.post(add_host_component_url.format(
                        ambari_public_ip, name, hostname, component),
                        auth=('admin', 'admin'))
                    if result.status_code != 201:
                        LOG.warning(
                            'Create host_component command failed. {0}'.format(
                                result.text))
                        return False

        return True

    def _provision_cluster(self, name, cluster_spec, ambari_host, servers):
        #TODO(jspeidel): encapsulate in another class

        ambari_public_ip = ambari_host.management_ip
        ambari_private_ip = ambari_host.internal_ip

        LOG.info(
            ' Provisioning Cluster via Ambari Server: {0} ...'.format(
                ambari_public_ip))

        for server in servers:
            self._spawn(server.provision_ambari, ambari_private_ip)

        self._wait_for_host_registrations(len(servers), ambari_host)

        # add cluster
        if not self._add_cluster(ambari_public_ip, name):
            return False

        # add configurations to cluster
        if not self._add_configurations_to_cluster(cluster_spec,
                                                   ambari_public_ip, name):
            return False

        # add services
        if not self._add_services_to_cluster(cluster_spec, ambari_public_ip,
                                             name):
            return False

        # add components to services
        if not self._add_components_to_services(cluster_spec, ambari_public_ip,
                                                name):
            return False

        # add hosts and host_components
        if not self._add_hosts_and_components(cluster_spec, servers,
                                              ambari_public_ip, name):
            return False

        return True

    def _wait_for_host_registrations(self, num_hosts, ambari_host):
        LOG.info(
            'Waiting for all Ambari agents to register with server ...')

        url = 'http://{0}:8080/api/v1/hosts'.format(ambari_host.management_ip)
        result = None
        json_result = None

        #TODO(jspeidel): timeout
        while result is None or len(json_result['items']) < num_hosts:
            context.sleep(5)
            try:
                result = requests.get(url, auth=('admin', 'admin'))
                json_result = json.loads(result.text)

                # TODO(jspeidel): just for debug
                LOG.info('Registered Hosts: {0} of {1}'.format(
                    len(json_result['items']), num_hosts))
                for hosts in json_result['items']:
                    LOG.debug('Registered Host: {0}'.format(
                        hosts['Hosts']['host_name']))
            except requests.ConnectionError:
                #TODO(jspeidel): max wait time
                LOG.info('Waiting to connect to ambari server ...')

    def _determine_host_for_server_component(self, component, cluster_spec,
                                             servers):
        found_node_group = None
        node_groups = cluster_spec.node_groups
        for node_group in node_groups.values():
            if component in node_group.components:
                found_node_group = node_group.name

        for host in servers:
            if host.role == found_node_group:
                return host

        raise Exception(
            'Server component [{0}] not specified in configuration'.format(
                component))

    def _update_server_hosts_files(self, servers):
        LOG.info('Updating server hosts files ...')

        for server in servers:
            # should pass in the savanna context so that it make files mods
            # on servers
            self._spawn(server.update_hosts_file, servers)

    def _install_services(self, cluster_name, ambari_host):
        LOG.info('Installing required Hadoop services ...')

        install_url = 'http://{0}:8080/api/v1/clusters/{' \
                      '1}/services?ServiceInfo/state=INIT'.format(
                          ambari_host.management_ip, cluster_name)
        body = '{"ServiceInfo": {"state" : "INSTALLED"}}'

        result = requests.put(install_url, data=body, auth=('admin', 'admin'))

        if result.status_code == 202:
            #TODO(jspeidel) don't hard code request id
            success = self._wait_for_async_request(1, cluster_name,
                                                   ambari_host)
            if success:
                LOG.info("Install of Hadoop stack successful.")
            else:
                LOG.critical('Install command failed.')
                raise RuntimeError('Hadoop service install failed')
        else:
            LOG.critical(
                'Install command failed. {0}'.format(result.text))
            raise RuntimeError('Hadoop service install failed')

        return success

    def _wait_for_async_request(self, request_id, cluster_name, ambari_host):
        request_url = 'http://{0}:8080/api/v1/clusters/{1}/requests/{' \
                      '2}/tasks?fields=Tasks/status'.format(
                          ambari_host.management_ip, cluster_name, request_id)
        started = False
        while not started:
            result = requests.get(request_url, auth=('admin', 'admin'))
            LOG.debug(
                'async request ' + request_url + ' response:\n' + result.text)
            json_result = json.loads(result.text)
            started = True
            for items in json_result['items']:
                status = items['Tasks']['status']
                if status == 'FAILED' or status == 'ABORTED':
                    return False
                else:
                    if status != 'COMPLETED':
                        started = False

            context.sleep(5)
        return started

    def update_ganglia_configurations(self, ganglia_server_ip, servers):
        for server in servers:
            self._spawn(server._configure_ganglia, ganglia_server_ip)

    def _start_services(self, cluster_name, ambari_host):
        LOG.info('Starting Hadoop services ...')
        LOG.info('Cluster name {0}, Ambari server ip {1}'
                 .format(cluster_name, ambari_host.management_ip))
        start_url = 'http://{0}:8080/api/v1/clusters/{' \
                    '1}/services?ServiceInfo/state=INSTALLED'\
            .format(ambari_host.management_ip, cluster_name)
        body = '{"ServiceInfo": {"state" : "STARTED"}}'

        result = requests.put(start_url, data=body, auth=('admin', 'admin'))
        if result.status_code == 202:
            # don't hard code request id
            success = self._wait_for_async_request(2, cluster_name,
                                                   ambari_host)
            if success:
                LOG.info(
                    "Successfully started Hadoop cluster '{0}'.".format(
                        cluster_name))
            else:
                LOG.critical('Failed to start Hadoop cluster.')
                raise RuntimeError('Failed to start Hadoop cluster.')

        else:
            LOG.critical(
                'Start command failed. Status: {0}, response: {1}'.
                format(result.status_code, result.text))
            raise RuntimeError('Hadoop cluster start failed.')

    def _get_default_cluster_configuration(self):
        with open(os.path.join(os.path.dirname(__file__), 'resources',
                               'default-cluster.template'), 'r') as f:
            return clusterspec.ClusterSpec(f.read())

    def _set_cluster_info(self, cluster, cluster_spec, hosts):
        info = cluster.info

        try:
            jobtracker_ip = self._determine_host_for_server_component(
                'JOBTRACKER', cluster_spec, hosts).management_ip
        except Exception:
            pass
        else:
            info['MapReduce'] = {
                'Web UI': 'http://%s:50030' % jobtracker_ip
            }

        try:
            namenode_ip = self._determine_host_for_server_component(
                'NAMENODE', cluster_spec, hosts).management_ip
        except Exception:
            pass
        else:
            info['HDFS'] = {
                'Web UI': 'http://%s:50070' % namenode_ip
            }

        try:
            ambari_ip = self._determine_host_for_server_component(
                'AMBARI_SERVER', cluster_spec, hosts).management_ip
        except Exception:
            pass
        else:
            info['Ambari Console'] = {
                'Web UI': 'http://%s:8080/#/main' % ambari_ip
            }

    # SAVANNA PLUGIN SPI METHODS:
    def configure_cluster(self, cluster):
        # take the user inputs from the cluster and node groups and convert
        # to a ambari blueprint
        processor = bp.BlueprintProcessor(json.load(
            open(os.path.join(os.path.dirname(__file__), 'resources',
                              'default-cluster.template'), "r")))
        processor.process_user_inputs(cluster.cluster_configs)
        processor.process_node_groups(cluster.node_groups)
        # NOTE: for the time being we are going to ignore the node group
        # level configurations.  we are not currently
        # defining node level configuration items (i.e. scope='cluster' in
        # all cases for returned configs)

        #create a cloud context

        #TODO(jmaron):  is base host name really necessary any longer?
        #cloud_ctx = ClusterContext(None, LOG)
        #self._add_instances_to_cluster_context (cloud_ctx, cluster)

        self.create_cluster(cluster, json.dumps(processor.blueprint))

    def get_versions(self):
        return ['1.3.0']

    def get_configs(self, hadoop_version):
        config_resource = cfg.ConfigurationProvider(
            json.load(open(os.path.join(os.path.dirname(__file__), 'resources',
                                        'ambari-config-resource.json'), "r")))
        return config_resource.get_config_items()

    # cluster name argument supports the non-savanna based cluster
    # creation mode
    def start_cluster(self, cluster, cluster_name=None):
        if cluster_name is None:
            cluster_name = cluster.name

        self._start_services(cluster_name,
                             self.cluster_name_to_ambari_host_mapping[
                                 cluster_name])

    def get_title(self):
        return 'Hortonworks Data Platform'

    def setup(self, conf):
        super(AmbariPlugin, self).setup(conf)

    def get_description(self):
        return 'The Hortonworks OpenStack plugin works with project ' \
               'Savanna to automate the deployment of the Hortonworks data' \
               ' platform on OpenStack based public & private clouds'



# Copyright (c) 2013 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import telnetlib

from savanna.tests.integration import base


class RestApiVersionsTest(base.ITestCase):

    def setUp(self):
        super(RestApiVersionsTest, self).setUp()
        telnetlib.Telnet(self.host, self.port)

    def test_version(self):
        """This test checks Savanna start
        """
        version_data = self.get_object(self.url_version, '', 200)
        get_version_data = {'versions': [{'status': 'CURRENT', 'id': 'v1.0'}]}
        self.assertEquals(version_data, get_version_data)



# Copyright (c) 2013 Hortonworks, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from savanna.plugins.hdp import baseprocessor as bp
from savanna.plugins import provisioning as p


class ConfigurationProvider(bp.BaseProcessor):
    config = None
    config_mapper = {}
    config_items = []

    def __init__(self, config):
        self.config = config
        self._initialize(config)

    def get_config_items(self):
        return self.config_items

    def get_applicable_target(self, name):
        return self.config_mapper.get(name)

    # temp method:  will not be required when all config values are researched
    def _get_target(self, applicable_target):
        if applicable_target == 'TODO':
            return 'general'

        return applicable_target

    def _initialize(self, config):
        for configuration in self.config['configurations']:
            for service_property in configuration['properties']:
                config = p.Config(service_property['name'],
                                  service_property['applicable_target'],
                                  service_property['scope'],
                                  config_type=
                                  service_property['config_type'],
                                  default_value=service_property
                                  ['default_value'],
                                  is_optional=service_property[
                                      'is_optional'],
                                  description=service_property[
                                      'description'])
                self.config_items.append(config)
                self.config_mapper[service_property['name']] = \
                    self._get_target(service_property['applicable_target'])
